/* 
 * FreeSWITCH Modular Media Switching Software Library / Soft-Switch Application
 * Copyright (C) 2005/2006, Anthony Minessale II <anthmct@yahoo.com>
 *
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is FreeSWITCH Modular Media Switching Software Library / Soft-Switch Application
 *
 * The Initial Developer of the Original Code is
 * Anthony Minessale II <anthmct@yahoo.com>
 * Portions created by the Initial Developer are Copyright (C)
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * 
 * Anthony Minessale II <anthmct@yahoo.com>
 *
 *
 * mod_rtctimer.c -- RTC Timer Module
 *
 */
#include <switch.h>
#include <stdio.h>
#include <linux/rtc.h>
#include <sys/ioctl.h>
#include <sys/time.h>
#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>

static const char modname[] = "mod_rtctimer";

struct rtc_timer_private {
	int flags;
	int fd;
	unsigned long rate;
};

static switch_status rtc_timer_init(switch_timer *timer)
{
	struct rtc_timer_private *private;
	if ((private = switch_core_alloc(timer->memory_pool, sizeof(*private)))) {
		private->flags = 1;
		if ((private->fd = open("/dev/rtc", O_RDONLY)) > -1) {
			private->rate = (timer->interval / 10) * 8;
			timer->private = private;
			ioctl(private->fd, RTC_IRQP_SET, private->rate);
			ioctl(private->fd, RTC_PIE_ON, 0);
			return SWITCH_STATUS_SUCCESS;
		}
	}

	return SWITCH_STATUS_GENERR;
}

static switch_status rtc_timer_next(switch_timer *timer)
{
	struct rtc_timer_private *private = timer->private;
	unsigned long data;
	int retval;

	if ((retval = read(private->fd, &data, sizeof(unsigned long))) < 0) {
		return SWITCH_STATUS_GENERR;
	}

	return SWITCH_STATUS_SUCCESS; 
}

static switch_status rtc_timer_destroy(switch_timer *timer)
{
	struct rtc_timer_private *private = timer->private;

	if (private->fd > -1) {
		ioctl(private->fd, RTC_UIE_OFF, 0);
		close(private->fd);
		private->fd = -1;
	}
	timer->private = NULL;
	return SWITCH_STATUS_SUCCESS;
}

static const switch_timer_interface rtc_timer_interface = {
	/*.interface_name*/		"rtc",
	/*.timer_init*/			rtc_timer_init,
	/*.timer_next*/			rtc_timer_next,
	/*.timer_destroy*/		rtc_timer_destroy
};

static const switch_loadable_module_interface mod_timers_module_interface = {
	/*.module_name*/			modname,
	/*.endpoint_interface*/		NULL,
	/*.timer_interface*/		&rtc_timer_interface,
	/*.dialplan_interface*/		NULL,
	/*.codec_interface*/		NULL,
	/*.application_interface*/	NULL
};

switch_status switch_module_load(const switch_loadable_module_interface **interface) {
	/* connect my internal structure to the blank pointer passed to me */
	*interface = &mod_timers_module_interface;

	/* indicate that the module should continue to be loaded */
	return SWITCH_STATUS_SUCCESS;
}

